/***************************************************************************
 *   Copyright (C) 2006-2011 by Julien Lemoine, Simon Viennot              *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tracewidget.h"
#include "trace.h"

#include <QtGui>

QColor colorFrom(QString colorName) {
	if(colorName == QString("transparent")){
		return QColor(255,255,255,0);	//alpha=0 : no color
	} else
	if(colorName == QString("lightred")){
		return QColor(255,127,127);	//red
	} else
	if (colorName == QString("green")) {
		return QColor(127,255,127);	//green
	} else
	if (colorName == QString("light green")) {
		return QColor(127,255,127,150);
	} else
	if(colorName == QString("gray")){
		return QColor(127,127,127);	//gray
	} else
	if(colorName == QString("blue")) {
		return QColor(127,127,255);	//blue
	} else
	if(colorName == QString("yellow")){
		return QColor(255,255,127);	//yellow
	}
	
	//default case : Qt will search the name in the predefined colors
	return QColor(colorName);
}

//indicate the background color depending on the number of unknown children
QColor colorUnknown(int unknownNumber) {
	switch(unknownNumber) {
		//red, green, blue, alpha (default =255)
		case 1  : return QColor(255,127,127,255); //red
		case 2  : return QColor(255,196,127,255); //orange
		case 3  : return QColor(255,255,127,255); //yellow
		case 4  : return QColor(255,255,127,255); //yellow
		case 5  : return QColor(127,255,127,255); //green
		case 6  : return QColor(127,255,127,255); //green
		case 7  : return QColor(127,255,127,200); //light green
		case 8  : return QColor(127,255,127,150); //light green
		case 9  : return QColor(127,255,127,100); //light green
		case 10 : return QColor(127,255,127, 50); //light green
		default : return colorFrom(QString("transparent")); //all other cases (including unknownNumber==0)
	}
}

//--------------------------------------------------------------------------
//Widget Trace : main trace to display the computing branch
//--------------------------------------------------------------------------

//initialization of main trace table
WidgetTrace::WidgetTrace(QWidget *parent) : QWidget( parent) {
	QVBoxLayout *vboxLayout = new QVBoxLayout(this);
	vboxLayout->setMargin(0);
	table=new QTableWidget(global::Max_Level, 5, this);
	table->setEnabled(false);
	
	QStringList titles;
	titles << "Index  " << "Traversal" << "Alive" << "Lives" << "Position";
	table->setHorizontalHeaderLabels(titles);
	
	int i, j;
	//initialization of columns width
	for(j=0; j<4; j++) {
		table->resizeColumnToContents(j);
	}
	table->setColumnWidth(4,500);
	//initialization with empty items
	for(i=0; i<global::Max_Level; i++) {
		table->setRowHeight(i,25);
		for(j=0; j<5; j++) {
			QTableWidgetItem *item = new QTableWidgetItem(tr("%1").arg( (i+1)*(j+1) ));
			item->setText("");
			table->setItem(i, j, item);
		}
	}
	
	table->setColumnHidden(1, !NodeStore::useTraversal); //hide the column if traversal is not used
	table->setEditTriggers ( QAbstractItemView::NoEditTriggers );  //no edit of the table allowed
	vboxLayout->addWidget(table);  //add the table to the layout
	
	//click on the table
	connect( table, SIGNAL( itemClicked(QTableWidgetItem*) ), this, SLOT( click_on_position(QTableWidgetItem*) ) );
	
	children_popup=new WidgetChildren(this); //table to trace children
}

//refresh the trace of the computing branch
void WidgetTrace::update_display() {
	QMutexLocker locker(&(Trace::trace_lock));
	
	//----------------update children table if displayed----------------------
	if(children_popup->isVisible()) children_popup->update_display();
	
	table->setColumnHidden(1, !NodeStore::useTraversal); //hide the column if traversal is not used
	QTableWidgetItem * item;
	
	//----------------update main table if displayed--------------------------
	if(!table->isVisible()) return;
	
	list<NodeTrace> ntrace = NodeStore::getMainTrace();
	list<NodeTrace>::iterator NTi=ntrace.begin();
	int row;
	for(row=0; row<global::Max_Level; row++) {
		if(NTi!=ntrace.end()) {
			NodeTrace n = *NTi;
			item=table->item(row,0);
			item->setData(Qt::DisplayRole, n.index);
			
			//information about the traversal of the tree
			item=table->item(row,1);
			item->setData(Qt::DisplayRole, n.traversal_info);
			
			//number of still alive positions
			item=table->item(row,2);
			item->setData(Qt::DisplayRole, QString("%1 ").arg(n.alive, 1, 10 ));
			item->setBackgroundColor( colorUnknown(n.alive) );
			
			//lives
			item=table->item(row,3);
			item->setData(Qt::DisplayRole, n.nb_lives);
			
			//position
			item=table->item(row,4);
			item->setData(Qt::DisplayRole, n.pos);
			item->setBackgroundColor( colorFrom(n.posColor));
			
			NTi++;
		} else {
			//there's nothing to display (end of list or level not yet filled with positions)
			item=table->item(row,0);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,1);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,2);
			item->setBackgroundColor( colorFrom(QString("transparent")) );
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,3);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,4);
			item->setData(Qt::DisplayRole, "");
			item->setBackgroundColor( colorFrom(QString("transparent")) );
		}
	}
}

//Function called after a click in the trace widget
//click on left column : zap child
//click on the position cell : display children
void WidgetTrace::click_on_position( QTableWidgetItem* item ) {
	if ( !item ) return;
	
	table->setItemSelected(item, FALSE );
	
	//nothing to do if it is a click trying to change the first position
	if(table->row(item)==0 && table->column(item)<3) return;
	
	//-----click on a left column-----------------
	//go to the next or the previous branch of the level
	if (table->column(item)<3) {
		Trace::request_level( table->row(item), table->column(item)==2 );
		return;
	}
	
	//-----click on a right column----------------
	QMutexLocker locker(&(Trace::trace_lock));
	
	int parent_candidate_level=table->row(item);
	
	//we do nothing if the click was too low in the interface
	if(parent_candidate_level >= Trace::current_level -1) return;
	
	//a position cell was clicked, we display children
	//the clicked position becomes the main position of the children table
	//we deduce the storeID of the position from the clicked line
	
	//update children tab
	children_popup->requestedId = NodeStore::getLevelStoreId(parent_candidate_level);
	children_popup->update_display();
	emit requestExplorer();
}

//init the displayed child id (used before beginning a computation)
void WidgetTrace::initialize() {
	children_popup->requestedId = 0;	//rootId of a computation is always 0
}


//--------------------------------------------------------------------------
//Widget Children : display all the available information about a given node
//--------------------------------------------------------------------------

//initialization of children trace table
WidgetChildren::WidgetChildren(QWidget *parent) : QWidget( parent) {
	setMinimumWidth( 750 );
	//setMinimumHeight( 600 );
	gridLayout = new QGridLayout(this);
	
	//table for the parents
	QLabel *title_parentTable = new QLabel;
	title_parentTable->setText("Parent nodes :");
	gridLayout->addWidget( title_parentTable, 1, 0, 1, 2 );
	
	nb_col=6;
	parentTable=new QTableWidget(1, nb_col, this); //1 row, 4 columns
	parentTable->setFixedHeight( 100 );
	initTable(parentTable);
	gridLayout->addWidget(parentTable, 2, 0, 1, 4);	//the table uses all 4 columns of the layout
	
	//table for the current node
	QLabel *title_currentTable = new QLabel;
	title_currentTable->setText("Current node :");
	gridLayout->addWidget( title_currentTable, 3, 0, 1, 2 );
	
	currentNodeTable=new QTableWidget(1, nb_col, this); //1 row, 4 columns
	currentNodeTable->setFixedHeight( 50 );
	initTable(currentNodeTable);
	gridLayout->addWidget(currentNodeTable, 4, 0, 1, 4);	//the table uses all 4 columns of the layout
	
	//table for the children
	QLabel *title_childrenTable = new QLabel;
	title_childrenTable->setText("Children nodes :");
	gridLayout->addWidget( title_childrenTable, 5, 0, 1, 2 );
	
	childrenTable=new QTableWidget(1, nb_col, this);
	initTable(childrenTable);
	gridLayout->addWidget(childrenTable, 6, 0, 1, 4);	//the table uses all 4 columns of the layout
	
	//connect the clicks on children and on parents
	connect( childrenTable, SIGNAL( itemClicked(QTableWidgetItem*) ),
	         this, SLOT( click_on_children(QTableWidgetItem*) ) );
	connect( parentTable, SIGNAL( itemClicked(QTableWidgetItem*) ),
	         this, SLOT( click_on_parent(QTableWidgetItem*) ) );
}

void WidgetChildren::initTable(QTableWidget *tableW) {
	QStringList titles;
	titles << "Lives" << "Traversal" << "Unknown" << "B.C.U." << "StoreId" << "Position";
	tableW->setHorizontalHeaderLabels(titles);
	
	//index of the column containing ids
	id_col = 4;
	
	//row height initialization
	tableW->setRowHeight(0,25);
	
	//column width initialization
	int j;
	for(j=0; j<nb_col; j++) {
		tableW->resizeColumnToContents(j);
	}
	tableW->setColumnWidth(nb_col-1,500);
	tableW->setColumnWidth(1,150);
	
	tableW->setColumnHidden(1, !NodeStore::useTraversal); //hide the column if traversal is not used
	tableW->setEditTriggers ( QAbstractItemView::NoEditTriggers ); //table is not editable
}

//refresh displayed children table
void WidgetChildren::update_display() {
	//note : no Trace lock, this function is always called only from an already locked function
	list<NodeTrace> ntrace = NodeStore::getChildrenTrace(requestedId);
	updateTable(childrenTable, ntrace);
	ntrace = NodeStore::getParentsTrace(requestedId);
	updateTable(parentTable, ntrace);
	ntrace = NodeStore::getOneNodeTrace(requestedId);
	if(!ntrace.empty()) {
		updateTable(currentNodeTable, ntrace);
	} else {
		//the node doesn't exist in NodeStore
		
	}
}

//TODO : enable/disable sorting
void WidgetChildren::updateTable(QTableWidget *tableW, list<NodeTrace> ntrace) {
	tableW->setColumnHidden(1, !NodeStore::useTraversal); //hide the column if traversal is not used
	//tableW->setSortingEnabled(false); //disable the sort
	int nodesNumber = ntrace.size();
	
	//update the size of the table
	while(nodesNumber < tableW->rowCount()) tableW->removeRow(0); //remove rows if needed
	while(nodesNumber > tableW->rowCount()) {
		tableW->insertRow(0); //add rows if needed
		tableW->setRowHeight(0,25); //row height initialization
	}
	
	//initialization with empty items
	//TODO : is the old item deleted ? risk of memory leak ?
	int i, j;
	for(i=0; i<nodesNumber; i++) {
		for(j=0; j<nb_col; j++) {
			QTableWidgetItem *item = new QTableWidgetItem(QString(""));
			tableW->setItem(i, j, item);
		}
	}
	
	list<NodeTrace>::iterator NTi;
	
	QTableWidgetItem * item;
	int row=0;
	int nbRows = tableW->rowCount();
	for(NTi=ntrace.begin(); NTi!=ntrace.end() && row < nbRows; NTi++) {
		NodeTrace n = *NTi;
		
		//lives
		item=tableW->item(row, 0);
		item->setData(Qt::DisplayRole, n.nb_lives);
		
		//traversal
		item=tableW->item(row, 1);
		item->setData(Qt::DisplayRole, n.traversal_info);
		
		//number of unknown children
		item=tableW->item(row, 2);
		item->setData(Qt::DisplayRole, QString("%1 ").arg(n.alive, 1, 10 ));
		item->setBackgroundColor( colorUnknown(n.alive) ); //color of the cell
		
		//B.C.U. : number of unknown children of the best child
		item=tableW->item(row, 3);
		if(n.bcu>0) {
			item->setData(Qt::DisplayRole, QString("%1 ").arg(n.bcu, 1, 10 ));
		} else {
			item->setData(Qt::DisplayRole, QString("?"));
		}
		item->setBackgroundColor( colorUnknown(n.bcu) ); //color of the cell
		
		//storeID
		item=tableW->item(row, 4);
		item->setData(Qt::DisplayRole, n.id);
		
		//position
		item=tableW->item(row, 5);
		item->setData(Qt::DisplayRole, n.pos);
		item->setBackgroundColor( colorFrom(n.posColor) );	//color
 
		//color dead nodes in gray
		if(!n.is_alive){
			for(i=0; i<=5; i++) {
				tableW->item(row, i)->setBackgroundColor(colorFrom(QString("gray")));
			}
		}
		
		row++;
	}
	//tableW->sortItems(3);
	//tableW->setSortingEnabled(true); //enable the sort
}

//function called when the children's table is clicked
void WidgetChildren::click_on_children( QTableWidgetItem* item ) {
	if ( !item ) return;
	childrenTable->setItemSelected(item, FALSE );
	
	//no click on an already dead node
	if( item->backgroundColor() == colorFrom(QString("gray")) ) return;
	
	//new requested id is in cell (row, id_col)
	int row=childrenTable->row(item);
	int clickedId = childrenTable->item(row, id_col)->data(Qt::DisplayRole).toInt();
	
	if (childrenTable->column(item)==4) {
		Trace::requestTraversalId( clickedId ); //click on the column of storeId = choose this node for traversal
	} else {
		requestedId =  clickedId ; //other columns = choose this node as the new current node of widgetchildren
	}
}

//function called when the parents table is clicked
void WidgetChildren::click_on_parent( QTableWidgetItem* item ) {
	if ( !item ) return;
	parentTable->setItemSelected(item, FALSE );
	int row=parentTable->row(item);
	int clickedId = parentTable->item(row, id_col)->data(Qt::DisplayRole).toInt();
	
	if (parentTable->column(item)==4) {
		Trace::requestTraversalId( clickedId ); //click on the column of storeId = choose this node for traversal
	} else {
		requestedId =  clickedId ; //other columns = choose this node as the new current node of widgetchildren
	}
}
