/***************************************************************************
 *   Copyright (C) 2006-2011 by Julien Lemoine, Simon Viennot              *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "glopwidget.h"
#include "childrentabwidget.h"

//for the function QColorFrom
#include "../computation/tracewidget.h"

#include "../baseclass/basegame.h"
#include "../baseclass/basenode.h"

#include <QtGui>

//---------------------ClickTextEdit---------------------

ClickTextEdit::ClickTextEdit( QWidget * parent ) : QTextEdit(parent) {
	//nothing to do
}

//double-click implementation for the children tab
void ClickTextEdit::mouseDoubleClickEvent( QMouseEvent * ) {
    // Emit content of the current line
    QTextCursor cursor = textCursor();
    cursor.movePosition(QTextCursor::StartOfLine, QTextCursor::MoveAnchor);
    cursor.movePosition(QTextCursor::EndOfLine, QTextCursor::KeepAnchor);
    QString content = cursor.selectedText();
	emit lineContent(content);
}

//------------------Children tab -------------------------------

void GlopWidget::setup_test_tab() {
	QWidget *test_tab = new QWidget;
	connect( this, SIGNAL(isComputing(bool)), test_tab, SLOT(setDisabled(bool)) );
	QVBoxLayout *test_tab_Vlayout = new QVBoxLayout( test_tab );
  //Children computation
	QGroupBox *children_group = new QGroupBox("Children computation", test_tab);
	test_tab_Vlayout->addWidget( children_group );
	QGridLayout *grid_test = new QGridLayout( children_group );
	
	QLabel *repr_2 = new QLabel( children_group );
	repr_2->setText("Position :");
	grid_test->addWidget( repr_2, 0, 0 );
	box_given_position_2 = new QLineEdit( children_group );
	grid_test->addWidget( box_given_position_2, 0, 1 );
	box_given_position_2->setText( QString("0*4.AB|0*4.AB") );
	box_given_position_2->setMinimumWidth( 150 );
	connect( box_given_position_2, SIGNAL(returnPressed()), this, SLOT(printChildren()) );
	
	QPushButton *back_button = new QPushButton( "Back to previous", children_group );
	back_button->setFixedSize(back_button->sizeHint());
	grid_test->addWidget( back_button, 1, 0 );
	connect( back_button, SIGNAL(clicked()), this, SLOT(previousChildren()) );
	
	QPushButton *children_button = new QPushButton( "Compute children", children_group );
	children_button->setFixedSize(children_button->sizeHint());
	grid_test->addWidget( children_button, 2, 0 );
	connect( children_button, SIGNAL(clicked()), this, SLOT(printChildren()) );
	
	textEdit_children = new ClickTextEdit( children_group );
	textEdit_children->setReadOnly( TRUE );
	connect(textEdit_children ,SIGNAL(lineContent(QString)), this, SLOT(nextChildren(QString)));
	grid_test->addWidget( textEdit_children, 1, 1, 3, 1);
	
	tabWidget->addTab( test_tab, "Children" );
}

//prints children of a given position
void GlopWidget::printChildren() {
	textEdit_children->setPlainText("");	//clean text box
	
	//add the position to the cache (used to go back to a previous position)
	positionCache.push_back(box_given_position_2->text()); 
	
	//currently selected nodeChoice
	int n = tabNodeWidget->currentIndex();
	Parameter::nodeChoice = n;
	
	//init the possible internal variables of this kind of node
	//note : it is usually used to initialize the first position of a computation
	//but it is also used by some nodes to initialize static variables
	BaseNode tmpNode(Parameter::nodeChoice, Parameter::computation_kind);
	tmpNode.initRootNode();
	
	//compute the children
	BaseGame g((box_given_position_2->text()).toStdString());
	g.computeChildrenSet();
	
	set<Line>::iterator it;
	list<Line> result;
	for(it=g.childrenSet().begin(); it!=g.childrenSet().end(); it++) {
		BaseGame h(*it);
		h.canonize();
		result.push_back(h.exportToLine());
	}
	
	result.sort();		//needed for unique
	result.unique();	//removes all consecutive duplicate positions
	
	//print children in the text box (use zero short notation)
	list<Line>::iterator Pi;
	for(Pi=result.begin();Pi!=result.end();Pi++){
		BaseGame h(*Pi);
		QString positionString = QString(h.exportToString().c_str());
		
		//print info about the current type of node
		BaseNode c(Parameter::nodeChoice, 0 /*useless value*/);
		QStringList info = c.infoAbout(h.exportToString());
		
		textEdit_children->setTextColor(Qt::black);
		if(info.size()>=2) {
			textEdit_children->setTextColor(colorFrom(info[1]));
		}
		
		textEdit_children->insertPlainText(positionString);
		
		if(!info.isEmpty()) {
			textEdit_children->insertPlainText(QString(" ") + info[0]);
		}
		textEdit_children->insertPlainText(QString("\n"));
	}
}

//compute the children of a given string
//called after a double-click in the children tab
void GlopWidget::nextChildren(QString position) {
	//keep only the position part (without nimber)
	int a = position.indexOf(' ');
	QString positionPart = position.left(a);
	
	//compute children (only if the positionPart is not empty)
	if(!positionPart.isEmpty()) {
		box_given_position_2->setText(positionPart);
		printChildren();
	}
}

//Go back to the previous position
void GlopWidget::previousChildren() {
	if(positionCache.size() >= 2){
		positionCache.pop_back(); //delete the current child
		QString previous = positionCache.back();
		positionCache.pop_back();
		box_given_position_2->setText(previous);
		printChildren();
	}
}
